import os
import re

from collections import deque
from itertools import imap
from operator import methodcaller

import pystache

from helga import settings
from helga.plugins.webhooks import HttpError, route


class Index(object):
    """
    Rendered object for the logger index page meant to show the full list
    of logged channels.
    """

    def title(self):
        return u'Channel Logs'

    def channels(self):
        log_dir = settings.CHANNEL_LOGGING_DIR
        lstrip = methodcaller('lstrip', '#')
        hidden = set(imap(lstrip, settings.CHANNEL_LOGGING_HIDE_CHANNELS))

        if not os.path.isdir(log_dir):
            raise StopIteration

        for chan in imap(lstrip, sorted(os.listdir(log_dir))):
            if chan in hidden:
                continue
            yield chan


class ChannelIndex(object):
    """
    Rendered object for the logger channel index page meant to show the full list
    of log files (UTC dates) for a given IRC channel.
    """

    def __init__(self, channel):
        self.channel = channel

    def title(self):
        return u'#{0} Channel Logs'.format(self.channel)

    def dates(self):
        channel = '#{0}'.format(self.channel)
        basedir = os.path.join(settings.CHANNEL_LOGGING_DIR, channel)

        if not os.path.isdir(basedir):
            raise HttpError(404)

        for date in sorted(os.listdir(basedir), reverse=True):
            yield date.replace('.txt', '')


class ChannelLog(object):
    """
    Rendered object for displaying the full contents of a channel log for a
    given channel and date.
    """

    def __init__(self, channel, date):
        self.channel_name = channel
        self.date = date
        self.logfile = '{0}.txt'.format(self.date)
        self.channel = '#{0}'.format(self.channel_name)

    @property
    def logfile_path(self):
        return os.path.join(settings.CHANNEL_LOGGING_DIR, self.channel, self.logfile)

    def title(self):
        """
        The page title
        """
        return u'{0} Channel Logs for {1}'.format(self.channel, self.date)

    def messages(self):
        """
        Generator for logged channel messages as a dictionary
        of the message time, message nick, and message contents
        """
        if not os.path.isfile(self.logfile_path):
            raise HttpError(404)

        line_pat = re.compile(r'^(\d{2}:?){3} - \w+ - .*$')
        message = ''
        log = deque()

        # XXX: This is kind of terrible. Some things will log only a single time
        # if the message sent over IRC has newlines. So we have to read in reverse
        # and construct the response list
        with open(self.logfile_path, 'r') as fp:
            for line in reversed(fp.readlines()):
                if not line_pat.match(line):
                    message = u''.join((line, message))
                    continue

                parts = line.strip().split(' - ')
                log.appendleft({
                    'time': parts.pop(0),
                    'nick': parts.pop(0),
                    'message': u'\n'.join((u' - '.join(parts), message)).rstrip('\n'),
                })
                message = ''

        if message:
            log.appendleft({
                'time': '',
                'nick': '',
                'message': message,
            })

        return log

    def download(self, request):
        """
        Offers this logfile as a download
        """
        request.setHeader('Content-Type', 'text/plain')
        request.setHeader('Content-Disposition',
                          'attachment; filename={0}'.format(self.logfile))
        with open(self.logfile_path, 'r') as fp:
            return '\n'.join(line.strip() for line in fp.readlines())


@route(r'/logger/?$')
@route(r'/logger/(?P<channel>[\w\-_]+)/?$')
@route(r'/logger/(?P<channel>[\w\-_]+)/(?P<date>[\w\-]+)(?P<as_text>\.txt)?/?$')
def logger(request, irc_client, channel=None, date=None, as_text=None):
    if not settings.CHANNEL_LOGGING:
        raise HttpError(501, 'Channel logging is not enabled')

    request.setHeader('Content-Type', 'text/html')
    renderer = pystache.renderer.Renderer(
        search_dirs=os.path.dirname(os.path.abspath(__file__))
    )

    if channel is None:
        page = Index()
    elif date is None:
        page = ChannelIndex(channel)
    else:
        page = ChannelLog(channel, date)
        if as_text is not None:
            return page.download(request)

    return renderer.render(page)



CHANNEL_LOGGING = True
CHANNEL_LOGGING_DIR = '/tmp/helga_channel_logs'
CHANNEL_LOGGING_HIDE_CHANNELS = ['#foo']
WEBHOOKS_CREDENTIALS = [('sduncan', 'sduncan')]
ENABLED_WEBHOOKS = ['logger']
COMMAND_ARGS_SHLEX = True



import os
import sys

SERVER = {
    'HOST': 'localhost',
    'PORT': 6667,
}

LOG_LEVEL = 'DEBUG'

# Helga plugin priority values
PLUGIN_PRIORITY_LOW = 25
PLUGIN_PRIORITY_NORMAL = 50
PLUGIN_PRIORITY_HIGH = 75

# Control the behavior of argument parsing for command plugins
# By default this is a naive str.split(' '), however a plugin
# may need this behavior to be a bit more robust. By setting this value
# to True, shlex.split() will be used instead so that commands
# like `helga foo bar "baz qux"` will yield an argument list like
# ['bar', 'baz qux'] instead of ['bar', '"baz', 'qux"']. Shlex splits
# will be the default and only supported behavior in a future version.
COMMAND_ARGS_SHLEX = False

# Channel logging. Set CHANNEL_LOGGING_DIR
CHANNEL_LOGGING = False
CHANNEL_LOGGING_DIR = '.logs'
CHANNEL_LOGGING_HIDE_CHANNELS = []

NICK = 'helga'
CHANNELS = [
    ('#bots',),
]

AUTO_RECONNECT = True
AUTO_RECONNECT_DELAY = 5
RATE_LIMIT = None

OPERATORS = ['sduncan']

# MongoDB settings
DATABASE = {
    'HOST': 'localhost',
    'PORT': 27017,
    'DB': 'helga',
}

TIMEZONE = 'US/Eastern'

# The default set of plugins enabled on any channel.
# By default, potentially noisy plugins are disabled
ENABLED_PLUGINS = [
    'dubstep',
    'facts',
    'help',
    'jira',
    'loljava',
    'manager',
    'meant_to_say',
    'oneliner',
    'operator',
    'poems',
    'reminders',
    'reviewboard',
    'stfu',
    'webhooks',
    'wiki_whois',

    # Sometimes, giphy may give back a gif of questionable content
    # 'giphy',

    # These can get super annoying in public channels
    # 'icanhazascii',

    # Generally, olga isn't being used
    # 'no_more_olga',
]

# A list of webhook names that should be enabled on process startup.
# If this value is None, then all webhooks loaded via entry points
# are run. An empty list will not load any webhooks
ENABLED_WEBHOOKS = None

# Set to False if all responses returned by plugins should be returned
# over IRC. If True, the first responding plugin will send a response
PLUGIN_FIRST_RESPONDER_ONLY = True

# If set to True, a command can be run by asking directly.
# For example `helga foo`
COMMAND_PREFIX_BOTNICK = True

# If non-empty, this char can be used to invoke a command without
# requiring the bot's nick. For example `helga foo` could be run with
# `!foo` instead
COMMAND_PREFIX_CHAR = '!'

# MISC PLUGIN SETTINGS
FACTS_REQUIRE_NICKNAME = False

# Jira settings. If JIRA_SHOW_FULL_DESCRIPTION is false, only links to the Jira
# ticket will be shown. Otherwise, the ticket title will be pulled and shown.
# Full descriptions require JIRA_REST_API to be set.
# JIRA_USERNAME and JIRA_PASSWORD are optional if authentication is required
JIRA_URL = 'http://localhost/{ticket}'
JIRA_REST_API = ''
JIRA_SHOW_FULL_DESCRIPTION = False
JIRA_AUTH = ('', '')

REVIEWBOARD_URL = 'http://localhost/{review}'
WIKI_URL = 'http://localhost/{user}'

# WEBHOOKS SETTINGS
WEBHOOKS_PORT = 8080
WEBHOOKS_CREDENTIALS = []  # Tuples of (user, pass)


def configure(overrides):
    """
    Applies custom configuration to global helga settings. Overrides can either be
    a python import path string like 'foo.bar.baz' or a filesystem path like
    'foo/bar/baz.py'
    """
    this = sys.modules[__name__]

    # Filesystem path to settings file
    if os.path.isfile(overrides):
        execfile(overrides, this.__dict__)
        return

    # Module import path settings file
    fromlist = [overrides.split('.')[-1]]
    overrides = __import__(overrides, this.__dict__, {}, fromlist)

    for attr in filter(lambda x: not x.startswith('_'), dir(overrides)):
        setattr(this, attr, getattr(overrides, attr))
